@@ -12,7 +12,7 @@ class TemplateGenerator(PathGenerator, ABC):
 
     def __init__(self, pattern: Pattern):
         self.log = logging.getLogger(__name__)
-        self.log.info("Creating template generator with template: %s", pattern)
+        self.log.debug("Creating template generator with template: %s", pattern)
         self.pattern = pattern
 
     def reset(self):
@@ -20,7 +20,9 @@ class TemplateGenerator(PathGenerator, ABC):
 
     def generate_replacement(self, file: File) -> str:
         self.log.debug("Rendering template for '%s'", file.relative_path)
-        return self.pattern.process(file.relative_path)
+        rendered_template = self.pattern.process(file)
+        self.log.debug("Rendered template: '%s'", rendered_template)
+        return rendered_template
 
 
 class TemplateNameGenerator(TemplateGenerator):
