@@ -2,7 +2,7 @@ import importlib
 import inspect
 import logging
 import pkgutil
-from functools import reduce, wraps
+from functools import reduce
 from logging import Logger
 from types import ModuleType
 from typing import Dict, List, Mapping, Optional, Tuple, Type, Union
@@ -20,6 +20,7 @@ from .tree_elements import (
     RawText,
     Tag,
     TagFactory,
+    TagFactoryFromClass,
     TagInstance,
     TagPlaceholder,
 )
@@ -179,7 +180,13 @@ class _TreeVisitor(TagTemplateParserVisitor):
 
 
 class TagTreeBuilder:
+    log: logging.Logger
+
+    def __init__(self):
+        self.log = logging.getLogger(self.__class__.__name__)
+
     def parse(self, text: str) -> Pattern:
+        self.log.debug("Parsing '%s'", text)
         lexer = TagTemplateLexer(InputStream(text))
         token_stream = CommonTokenStream(lexer)
         token_stream.fill()
@@ -277,43 +284,63 @@ class ConfigurationError(TagError):
         super().__init__(location, tag_name, f"Configuration not valid: {message}")
 
 
+class TagCategory:
+    log: Logger
+
+    name: str
+    description: Optional[str] = None
+    tag_map: Dict[str, TagFactory]
+
+    def __init__(self, name: str, description: Optional[str] = None):
+        self.log = logging.getLogger(self.__class__.__name__)
+        self.tag_map = {}
+        self.name = name
+        self.description = description
+
+    def register_tag(self, tag_class: Type[Tag], tag_name: Optional[str] = None):
+        _simple_tag_factory = TagFactoryFromClass(tag_class, tag_name)
+        self.log.debug(
+            f"Registering class {tag_class} as {_simple_tag_factory.tag_name} tag"
+        )
+        self.register_tag_factory(_simple_tag_factory, tag_name)
+
+    def register_tag_factory(
+        self, tag_factory: TagFactory, tag_name: Optional[str] = None
+    ):
+        if tag_name is None:
+            tag_name = tag_factory.tag_name
+        if not tag_name:
+            raise ValueError(f"Invalid tag name '{repr(tag_name)}'")
+        if tag_name in self.tag_map:
+            raise ValueError(f"Factory for tag '{tag_name}' already registered")
+        self.tag_map[tag_name] = tag_factory
+
+    def find_tag_factory(self, tag_name: str) -> Optional[TagFactory]:
+        return self.tag_map.get(tag_name, None)
+
+
 class TagRegistry:
     log: Logger
     _tag_class_suffix = "Tag"
-    tag_registry: Dict[str, TagFactory]
+    category_map: Dict[str, TagCategory]
 
     def __init__(self):
-        self.log = logging.getLogger(__name__)
-        self.tag_registry = {}
+        self.log = logging.getLogger(self.__class__.__name__)
+        self.category_map = {}
 
-    def register_tag(self, tag_class: Type[Tag], tag_name: Optional[str] = None):
-        if not tag_name:
-            tag_class_name = tag_class.__name__
-            if tag_class_name.endswith(self._tag_class_suffix):
-                tag_name = tag_class_name[: -len(self._tag_class_suffix)]
-            else:
-                raise ValueError(
-                    f"Could not determine tag name from tag class: {tag_class_name}"
-                )
+    @property
+    def categories(self) -> List[str]:
+        return sorted(self.category_map.keys())
 
-        def _simple_tag_factory(*args, **kwargs):
-            tag = tag_class()
-            tag.configure(*args, **kwargs)
-            return tag
-
-        _simple_tag_factory.__doc__ = tag_class.__doc__
-        self.log.debug(f"Registering class {tag_class} as {tag_name} tag")
-        self.register_tag_factory(_simple_tag_factory, tag_name)
-
-    def register_tag_factory(self, tag_factory: TagFactory, tag_name: str):
-        if not tag_name:
-            raise ValueError(f"Invalid tag name '{tag_name}'")
-        if tag_name in self.tag_registry:
-            raise ValueError(f"Factory for tag '{tag_name}' already registered")
-        self.tag_registry[tag_name] = tag_factory
+    def find_category(self, category_name: str) -> Optional[TagCategory]:
+        return self.category_map.get(category_name, None)
 
     def find_tag_factory(self, tag_name: str) -> Optional[TagFactory]:
-        return self.tag_registry.get(tag_name, None)
+        for category in self.category_map.values():
+            tag_factory = category.find_tag_factory(tag_name)
+            if tag_factory:
+                return tag_factory
+        return None
 
     def bind(self, pattern: Pattern) -> Pattern:
         return self._rewrite_pattern(pattern)
@@ -363,6 +390,15 @@ class TagRegistry:
 
         return TagInstance(tag, context=context_pattern)
 
+    def register_category(
+        self, category_name: str, description: Optional[str] = None
+    ) -> TagCategory:
+        if self.find_category(category_name) is not None:
+            raise ValueError(f"Category '{category_name}' already registered")
+        new_category = TagCategory(category_name)
+        self.category_map[category_name] = new_category
+        return new_category
+
     def register_tags_in_module(self, module: ModuleType):
         self.log.debug(f"Discovering tags in module '{module}'")
 
@@ -371,8 +407,15 @@ class TagRegistry:
                 return False
             return klass.__name__.endswith(self._tag_class_suffix)
 
+        if module.__package__:
+            category_name = module.__name__[len(module.__package__) + 1 :]
+        else:
+            category_name = module.__name__
+
+        # TODO: do not register empty modules
+        module_category = self.register_category(category_name)
         for _, tag_class in inspect.getmembers(module, is_tag_class):
-            self.register_tag(tag_class)
+            module_category.register_tag(tag_class)
 
     def register_tags_in_package(self, package):
         self.log.debug(f"Discovering tags in package '{package}'")
